@@ -261,7 +261,7 @@ class SignalInstance:
 
     def __init__(
         self,
-        signature: Signature = Signature(),
+        signature: Union[Signature, Tuple] = Signature(),
         *,
         instance: Any = None,
         name: Optional[str] = None,
@@ -298,14 +298,14 @@ class SignalInstance:
         return self._instance
 
     @property
-    def name(self) -> Optional[str]:
+    def name(self) -> str:
         """Name of this `SignalInstance`."""
         return self._name or ""
 
     def __repr__(self) -> str:
         """Return repr."""
         name = f" {self.name!r}" if self.name else ""
-        instance = f" on {self.instance!r}" if self.instance else ""
+        instance = f" on {self.instance!r}" if self.instance is not None else ""
         return f"<{type(self).__name__}{name}{instance}>"
 
     @overload
@@ -315,6 +315,7 @@ class SignalInstance:
         check_nargs: Optional[bool] = ...,
         check_types: Optional[bool] = ...,
         unique: Union[bool, str] = ...,
+        max_args: Optional[int] = None,
     ) -> Callable[[Callable], Callable]:
         ...  # pragma: no cover
 
@@ -326,6 +327,7 @@ class SignalInstance:
         check_nargs: Optional[bool] = ...,
         check_types: Optional[bool] = ...,
         unique: Union[bool, str] = ...,
+        max_args: Optional[int] = None,
     ) -> Callable:
         ...  # pragma: no cover
 
@@ -336,6 +338,7 @@ class SignalInstance:
         check_nargs: Optional[bool] = None,
         check_types: Optional[bool] = None,
         unique: Union[bool, str] = False,
+        max_args: Optional[int] = None,
     ) -> Union[Callable[[Callable], Callable], Callable]:
         """Connect a callback ("slot") to this signal.
 
@@ -367,6 +370,10 @@ class SignalInstance:
             connected.  If the literal string "raise" is passed to `unique`, then a
             `ValueError` will be raised if the slot is already connected.
             By default `False`.
+        max_args : int, optional
+            If provided, `slot` will be called with no more more than `max_args` when
+            this SignalInstance is emitted.  (regardless of how many arguments are
+            emitted).
 
         Raises
         ------
@@ -383,7 +390,7 @@ class SignalInstance:
         if check_types is None:
             check_types = self._check_types_on_connect
 
-        def _wrapper(slot: Callable) -> Callable:
+        def _wrapper(slot: Callable, max_args: Optional[int] = max_args) -> Callable:
             if not callable(slot):
                 raise TypeError(f"Cannot connect to non-callable object: {slot}")
 
@@ -396,16 +403,16 @@ class SignalInstance:
                         )
                     return slot
 
-                slot_sig = maxargs = None
-                if check_nargs:
-                    slot_sig, maxargs = self._check_nargs(slot, self.signature)
+                slot_sig = None
+                if check_nargs and (max_args is None):
+                    slot_sig, max_args = self._check_nargs(slot, self.signature)
                 if check_types:
                     slot_sig = slot_sig or signature(slot)
                     if not _parameter_types_match(slot, self.signature, slot_sig):
                         extra = f"- Slot types {slot_sig} do not match types in signal."
                         self._raise_connection_error(slot, extra)
 
-                self._slots.append((self._normalize_slot(slot), maxargs))
+                self._slots.append((_normalize_slot(slot), max_args))
             return slot
 
         return _wrapper(slot) if slot else _wrapper
@@ -444,6 +451,20 @@ class SignalInstance:
             If this is not a single-value signal
         AttributeError
             If `obj` has no attribute `attr`.
+
+        Examples
+        --------
+        >>> class T:
+        ...     sig = Signal(int)
+        ...
+        >>> class SomeObj:
+        ...     x = 1
+        ...
+        >>> t = T()
+        >>> my_obj = SomeObj()
+        >>> t.sig.connect_setattr(my_obj, 'x')
+        >>> t.sig.emit(5)
+        >>> assert my_obj.x == 5
         """
         ref = obj if isinstance(obj, weakref.ref) else weakref.ref(obj)
         if not hasattr(ref(), attr):
@@ -499,15 +520,22 @@ class SignalInstance:
 
         Also returns the maximum number of arguments that we can pass to the slot
         """
-        try:
-            slot_sig = signature(slot)
-        except ValueError as e:
-            warnings.warn(
-                f"{e}. To silence this warning, connect with " "`check_nargs=False`"
-            )
-            return None, None
+        maxargs: Optional[int]
+        str_sig = _guess_qtsignal_signature(slot)
+        if str_sig is not None and "(" in str_sig:
+            inner = str_sig.split("(", 1)[1].split(")", 1)[0]
+            minargs = maxargs = inner.count(",") + 1 if inner else 0
+            slot_sig = None
+        else:
+            try:
+                slot_sig = signature(slot)
+            except ValueError as e:
+                warnings.warn(
+                    f"{e}. To silence this warning, connect with " "`check_nargs=False`"
+                )
+                return None, None
+            minargs, maxargs = _acceptable_posarg_range(slot_sig)
 
-        minargs, maxargs = _acceptable_posarg_range(slot_sig)
         n_spec_params = len(spec.parameters)
         # if `slot` requires more arguments than we will provide, raise.
         if minargs > n_spec_params:
@@ -525,23 +553,11 @@ class SignalInstance:
         msg += f"\n\nAccepted signature: {self.signature}"
         raise ValueError(msg)
 
-    def _normalize_slot(self, slot: NormedCallback) -> NormedCallback:
-        if isinstance(slot, MethodType):
-            return _get_method_name(slot) + (None,)
-        if isinstance(slot, PartialMethod):
-            return _partial_weakref(slot)
-        if isinstance(slot, tuple) and not isinstance(slot[0], weakref.ref):
-            return (weakref.ref(slot[0]), slot[1], slot[2])
-        return slot
-
     def _slot_index(self, slot: NormedCallback) -> int:
         """Get index of `slot` in `self._slots`.  Return -1 if not connected."""
         with self._lock:
-            normed = self._normalize_slot(slot)
-            for i, (s, m) in enumerate(self._slots):
-                if s == normed:
-                    return i
-            return -1
+            normed = _normalize_slot(slot)
+            return next((i for i, s in enumerate(self._slots) if s[0] == normed), -1)
 
     def disconnect(
         self, slot: Optional[NormedCallback] = None, missing_ok: bool = True
@@ -573,6 +589,7 @@ class SignalInstance:
                 self._slots.pop(idx)
             elif not missing_ok:
                 raise ValueError(f"slot is not connected: {slot}")
+            # breakpoint()
 
     def __contains__(self, slot: NormedCallback) -> bool:
         """Return `True` if slot is connected."""
@@ -647,7 +664,7 @@ class SignalInstance:
                 raise TypeError(
                     f"Cannot emit args {args} from signal {self!r} with "
                     f"signature {self.signature}:\n{e}"
-                )
+                ) from e
 
         if check_types and not _parameter_types_match(
             lambda: None, self.signature, _build_signature(*(type(a) for a in args))
@@ -721,10 +738,11 @@ class SignalInstance:
                         if method is not None:
                             cb = method
                         else:
-                            cb = getattr(obj, name, None)
-                            if cb is None:  # pragma: no cover
+                            _cb = getattr(obj, name, None)
+                            if _cb is None:  # pragma: no cover
                                 rem.append(slot)  # object has changed?
                                 continue
+                            cb = _cb
                     else:
                         cb = slot
 
@@ -908,6 +926,16 @@ def _build_signature(*types: AnyType) -> Signature:
     return Signature(params)
 
 
+def _normalize_slot(slot: Union[Callable, NormedCallback]) -> NormedCallback:
+    if isinstance(slot, MethodType):
+        return _get_method_name(slot) + (None,)
+    if isinstance(slot, PartialMethod):
+        return _partial_weakref(slot)
+    if isinstance(slot, tuple) and not isinstance(slot[0], weakref.ref):
+        return (weakref.ref(slot[0]), slot[1], slot[2])
+    return slot
+
+
 # def f(a, /, b, c=None, *d, f=None, **g): print(locals())
 #
 # a: kind=POSITIONAL_ONLY,       default=Parameter.empty    # 1 required posarg
@@ -1027,7 +1055,7 @@ def _get_method_name(slot: MethodType) -> Tuple[weakref.ref, str]:
     # will not be equal to getattr(obj, obj.method.__name__).
     # We check for that case here and find the proper name in the function's closures
     if getattr(obj, slot.__name__, None) != slot:
-        for c in slot.__closure__ or ():  # type: ignore
+        for c in slot.__closure__ or ():
             cname = getattr(c.cell_contents, "__name__", None)
             if cname and getattr(obj, cname, None) == slot:
                 return weakref.ref(obj), cname
@@ -1043,6 +1071,42 @@ def _get_method_name(slot: MethodType) -> Tuple[weakref.ref, str]:
     return weakref.ref(obj), slot.__name__
 
 
+def _guess_qtsignal_signature(obj: Any) -> Optional[str]:
+    """Return string signature if `obj` is a SignalInstance or Qt emit method.
+
+    This is a bit of a hack, but we found no better way:
+    https://stackoverflow.com/q/69976089/1631624
+    https://bugreports.qt.io/browse/PYSIDE-1713
+    """
+    # on my machine, this takes ~700ns on PyQt5 and 8.7µs on PySide2
+    _repr = repr(obj)
+    if "built-in method emit of" in _repr:
+        if ".pyqtBoundSignal" in _repr:
+            return cast(str, obj.__self__.signal)
+
+        # we likely have the emit method of a SignalInstance
+        # call it with ridiculous params to get the err
+        return _ridiculously_call_emit(obj)
+    if "bound PYQT_SIGNAL" in _repr:
+        return cast(str, obj.signal)
+    if "QtCore.SignalInstance" in _repr and hasattr(obj, "emit"):
+        return _ridiculously_call_emit(obj.emit)
+    return None
+
+
+_CRAZY_ARGS = (1,) * 255
+
+
+def _ridiculously_call_emit(emitter: Any) -> Optional[str]:
+    """Call SignalInstance emit() to get the signature from err message."""
+    try:
+        emitter(*_CRAZY_ARGS)
+    except TypeError as e:
+        if "only accepts" in str(e):
+            return str(e).split("only accepts")[0].strip()
+    return None  # pragma: no cover
+
+
 try:
     import cython
 except ImportError:  # pragma: no cover
